
/*
>> PulseSensor_FastReplay <<
This code is derived from Pulse Sensor Amped by Joel Murphy and Yury Gitman
    www.pulsesensor.com
Code modifications have been made by Paul Christensen (www.potomacmeso.com) to allow a fast audio replay
of the heart beat rhythm and to output serial data to a Processing program, PulseSensor_IBIdistribution, that provides a display
of the statistics of the heart beat interbeat interval (IBI).
The EasyPulse sensor manufactured by Embedded Labs has been used to generate the heartbeat data used by this code.

Pulse Sensor sample aquisition and processing happens in the background via Timer 2 interrupt. 2mS sample rate.
PWM on pins 3 and 11 will not work when using this code, because we are using Timer 2!
The following variables are automatically updated:
Signal :    int that holds the analog signal data straight from the sensor. updated every 2mS.
IBI  :      int that holds the time interval between beats. 2mS resolution.
BPM  :      int that holds the heart rate value, derived every beat, from averaging previous 10 IBI values.
QS  :       boolean that is made true whenever Pulse is found and BPM is updated. User must reset.
Pulse :     boolean that is true when a heartbeat is sensed then false in time with pin13 LED going out.

All the work to find the heartbeat and determine the heartrate happens in the code below.
Pin 13 LED will blink with heartbeat.
If you want to use pin 13 for something else, adjust the interrupt handler
*/

//#include <EEPROM.h>
//  VARIABLES
int pulsePin = 0;                 // Pulse Sensor purple wire connected to analog pin 0
int blinkPin = 13;                // pin to blink led at each beat
int audioPin = 4;                 // to produce an accelerated audio pattern corresponding to the last collected set of beats
int interval[200];                //  interval between beats (ms).
int numBeats = 100;               //number of beats collected for replay
int count;

// these variables are volatile because they are used during the interrupt service routine!
volatile int BPM;                   // used to hold the pulse rate
volatile int Signal;                // holds the incoming raw data
volatile int IBI = 600;             // holds the time between beats, must be seeded! 
volatile boolean Pulse = false;     // true when pulse wave is high, false when it's low
volatile boolean QS = false;        // becomes true when Arduoino finds a beat.


void setup(){
  pinMode(blinkPin,OUTPUT);         // pin that will blink to your heartbeat!
  pinMode(audioPin,OUTPUT);         //earphone drive pin
  count = 0;
  Serial.begin(115200);             // we agree to talk fast to Processing!
  interruptSetup();                 // sets up to read Pulse Sensor signal every 2mS    
}

void loop(){
  sendDataToProcessing('S', Signal);     // send Processing the raw Pulse Sensor data

  if (QS == true){                       // Quantified Self flag is true when arduino finds a heartbeat
        sendDataToProcessing('B',BPM);   // send heart rate with a 'B' prefix
        sendDataToProcessing('Q',IBI);   // send time between beats with a 'Q' prefix
      if (count < numBeats){
        for (int i=numBeats-1; i>0; i--){   // shift the data in interval array 
        interval[i] = interval[i-1];     
        }
        interval[0] = IBI;       // first interval is new IBI
        Serial.println(IBI);      //debug
      }
    //  if (count == numBeats) Serial.println("OK");      //debug
    QS = false;                      // reset the Quantified Self flag for next time
    count = count + 1;    
   }
  
  delay(20);                             //  take a break
  
    if (Serial.available()>0){
      char rhythm = Serial.read();      //check for restart or replay commands from Processing                                        
 //     Serial.println(rhythm);
      if (rhythm == 'C') count = 0;  // Start gathering new data set
      
      if (rhythm == 'F' && count > numBeats){    //Fast play if data set complete 
//         Serial.println("start");             //debug
         for (int i=numBeats-1; i>0; i--){
//          Serial.print(i);   Serial.print("  ");        //debug
//          Serial.println(interval[i]);
            digitalWrite(audioPin, HIGH);          //generate earphone click
            delay(1);
            digitalWrite(audioPin, LOW);
          delay(interval[i]>>3);
         }
      }
    }
}


void sendDataToProcessing(char symbol, int data ){
    Serial.print(symbol);                // symbol prefix tells Processing what type of data is coming
    Serial.println(data);                // the data to send culminating in a carriage return
  }







